import BookTrackerPlugin from "@src/main";
import type { ReadingLogEntry } from "@src/types";
import { App } from "obsidian";

export class Storage {
	public constructor(
		private readonly app: App,
		private readonly plugin: BookTrackerPlugin
	) {}

	private getFilePath(filename: string): string {
		return `${this.plugin.manifest.dir!!}/${filename}`;
	}

	public async readJSON<T>(filename: string): Promise<T | null> {
		const filePath = this.getFilePath(filename);
		const content = await this.app.vault.adapter.read(filePath);
		if (!content) {
			return null;
		}

		try {
			return JSON.parse(content) as T;
		} catch (error) {
			console.error(`Error parsing JSON from ${filePath}:`, error);
			return null;
		}
	}

	public async writeJSON<T>(filename: string, data: T): Promise<void> {
		const filePath = this.getFilePath(filename);
		const content = JSON.stringify(data, null, 2);
		await this.app.vault.adapter.write(filePath, content);
	}
}

export class ReadingLog {
	private entries: ReadingLogEntry[] = [];

	public constructor(private readonly plugin: BookTrackerPlugin) {
		this.loadEntries().catch((error) => {
			console.error("Failed to load reading log entries:", error);
		});
	}

	private async loadEntries() {
		const entries = await this.plugin.storage.readJSON<ReadingLogEntry[]>(
			"reading-log.json"
		);
		if (entries) {
			this.entries = entries.map((entry) => ({
				...entry,
				createdAt: new Date(entry.createdAt),
			}));
		}
	}

	private sortEntries() {
		this.entries = this.entries.sort(
			(a, b) => b.createdAt.getTime() - a.createdAt.getTime()
		);
	}

	private async storeEntries() {
		this.sortEntries();
		await this.plugin.storage.writeJSON("reading-log.json", this.entries);
	}

	public getEntries(): ReadingLogEntry[] {
		return this.entries;
	}

	public getLatestEntry(book: string): ReadingLogEntry | null {
		const entriesForBook = this.entries.filter(
			(entry) => entry.book === book
		);

		return entriesForBook.length > 0
			? entriesForBook[entriesForBook.length - 1]
			: null;
	}

	public async addEntry(
		book: string,
		pageEnded: number,
		pageLength: number
	): Promise<void> {
		const latestEntry = this.getLatestEntry(book);

		const newEntry: ReadingLogEntry = {
			book,
			pagesRead: latestEntry
				? pageEnded - latestEntry.pagesReadTotal
				: pageEnded,
			pagesReadTotal: pageEnded,
			pagesRemaining: pageLength - pageEnded,
			createdAt: new Date(),
		};

		await this.addRawEntry(newEntry);
	}

	public async addRawEntry(entry: ReadingLogEntry) {
		this.entries.push(entry);
		await this.storeEntries();
	}

	public async removeEntries(book: string): Promise<void> {
		this.entries = this.entries.filter((entry) => entry.book !== book);
		await this.storeEntries();
	}

	public async removeLastEntry(book: string): Promise<void> {
		const latestEntryIndex = this.entries.findLastIndex(
			(entry) => entry.book === book
		);

		if (latestEntryIndex !== -1) {
			this.entries.splice(latestEntryIndex, 1);
			await this.storeEntries();
		}
	}

	public async updateEntry(i: number, entry: ReadingLogEntry): Promise<void> {
		this.entries[i] = entry;
		await this.storeEntries();
	}

	public async spliceEntry(i: number): Promise<ReadingLogEntry> {
		const entry = this.entries.splice(i, 1);
		await this.storeEntries();
		return entry[0];
	}

	public async clearEntries(): Promise<void> {
		this.entries = [];
		await this.storeEntries();
	}
}
